@@ -1,18 +1,20 @@
 from .driver import Driver
 from ..models.ops import Ops
+from ..models.machine import Machine
 
 
 class NoDeviceDriver(Driver):
     """
     A dummy driver that is used if the user has no machine.
     """
-    label = 'No driver'
-    subtitle = 'No connection'
+
+    label = _("No driver")
+    subtitle = _("No connection")
 
     async def connect(self) -> None:
         pass
 
-    async def run(self, ops: Ops) -> None:
+    async def run(self, ops: Ops, machine: Machine) -> None:
         pass
 
     async def set_hold(self, hold: bool = True) -> None:
