@@ -3,27 +3,30 @@ import logging
 import uuid
 from pathlib import Path
 from typing import List, Optional, Tuple, Dict
-from gi.repository import Gtk, Gio, GLib, Gdk, Adw  # type: ignore
+from gi.repository import Gtk, Gio, GLib, Gdk, Adw, GObject  # type: ignore
 from .. import __version__
 from ..tasker.context import ExecutionContext
-from ..config import config, task_mgr
+from ..config import config, task_mgr, machine_mgr
 from ..driver import get_driver_cls
 from ..driver.driver import driver_mgr, DeviceStatus
 from ..driver.dummy import NoDeviceDriver
 from ..util.resources import get_icon
+from ..models.machine import Machine
 from ..models.doc import Doc
 from ..models.workpiece import WorkPiece
 from ..opsencoder.gcode import GcodeEncoder
 from ..render import renderers, renderer_by_mime_type, renderer_by_extension
 from ..undo.list_cmd import ListItemCommand, ReorderListCommand
-from .workplanview import WorkPlanView
+from .workflowview import WorkflowView
 from .workbench.surface import WorkSurface
+from .layerlist import LayerListView
 from .statusview import (
     ConnectionStatusMonitor,
     TransportStatus,
     MachineStatusMonitor,
 )
 from .machineview import MachineView
+from .preferencesdialog import PreferencesWindow
 from .machinesettings import MachineSettingsDialog
 from .progress import TaskProgressBar
 from .workpieceprops import WorkpiecePropertiesWidget
@@ -47,7 +50,6 @@ css = """
 }
 
 .statusbar:hover {
-    /* A subtle highlight that works on both light and dark themes. */
     background-color: alpha(@theme_fg_color, 0.1);
 }
 
@@ -60,20 +62,49 @@ css = """
     padding: 6px 12px 6px 12px;
 }
 
-/* This is the new rule to make menu separators visible */
 .menu separator {
     border-top: 1px solid @borders;
     margin-top: 5px;
     margin-bottom: 5px;
 }
+
+.warning-label {
+    color: @warning_color;
+    font-weight: bold;
+}
 """
 
 
+# This allows the plain Python Machine object to be stored in a Gio.ListStore.
+class MachineListItem(GObject.Object):
+    __gtype_name__ = "MachineListItem"
+
+    def __init__(self, machine: Machine):
+        super().__init__()
+        self.machine = machine
+
+
 class MainWindow(Adw.ApplicationWindow):
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
         self.set_title(_("Rayforge"))
 
+        # The main content box
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
+
+        # The ToastOverlay will wrap the main content box
+        self.toast_overlay = Adw.ToastOverlay()
+        self.toast_overlay.set_child(vbox)
+
+        # Set the ToastOverlay as the window's content
+        self.set_content(self.toast_overlay)
+
+        # Add a global click handler to manage focus correctly.
+        root_click_gesture = Gtk.GestureClick.new()
+        root_click_gesture.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
+        root_click_gesture.connect("pressed", self._on_root_click_pressed)
+        self.add_controller(root_click_gesture)
+
         display = Gdk.Display.get_default()
         monitors = display.get_monitors()
 
@@ -101,11 +132,7 @@ class MainWindow(Adw.ApplicationWindow):
                 int(geometry.width * 0.6), int(geometry.height * 0.6)
             )
         else:
-            self.set_default_size(1200, 900)
-
-        # Create the main vbox
-        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
-        self.set_content(vbox)
+            self.set_default_size(1100, 800)
 
         # Setup keyboard actions.
         self._setup_actions()
@@ -144,13 +171,13 @@ class MainWindow(Adw.ApplicationWindow):
 
         # Import and export icons
         open_button = Gtk.Button()
-        open_button.set_child(get_icon("open"))
+        open_button.set_child(get_icon("document-open-symbolic"))
         open_button.set_tooltip_text(_("Import image"))
         open_button.connect("clicked", self.on_open_clicked)
         toolbar.append(open_button)
 
         self.export_button = Gtk.Button()
-        self.export_button.set_child(get_icon("publish"))
+        self.export_button.set_child(get_icon("document-save-symbolic"))
         self.export_button.set_tooltip_text(_("Generate G-code"))
         self.export_button.connect("clicked", self.on_export_clicked)
         toolbar.append(self.export_button)
@@ -170,13 +197,13 @@ class MainWindow(Adw.ApplicationWindow):
         toolbar.append(sep)
 
         clear_button = Gtk.Button()
-        clear_button.set_child(get_icon("clear-layers"))
+        clear_button.set_child(get_icon("edit-clear-all-symbolic"))
         clear_button.set_tooltip_text(_("Remove all workpieces"))
         clear_button.connect("clicked", self.on_clear_clicked)
         toolbar.append(clear_button)
 
-        self.visibility_on_icon = get_icon("visibility-on")
-        self.visibility_off_icon = get_icon("visibility-off")
+        self.visibility_on_icon = get_icon("stock-eye-symbolic")
+        self.visibility_off_icon = get_icon("eye-not-looking-symbolic")
         button = Gtk.ToggleButton()
         button.set_active(True)
         button.set_child(self.visibility_on_icon)
@@ -185,8 +212,8 @@ class MainWindow(Adw.ApplicationWindow):
         button.connect("clicked", self.on_button_visibility_clicked)
 
         # Camera Image Visibility Toggle Button
-        self.camera_visibility_on_icon = get_icon("camera-on")
-        self.camera_visibility_off_icon = get_icon("camera-off")
+        self.camera_visibility_on_icon = get_icon("camera-app-symbolic")
+        self.camera_visibility_off_icon = get_icon("camera-disabled-symbolic")
         self.camera_visibility_button = Gtk.ToggleButton()
         self.camera_visibility_button.set_active(True)
         self.camera_visibility_button.set_child(self.camera_visibility_on_icon)
@@ -200,7 +227,7 @@ class MainWindow(Adw.ApplicationWindow):
 
         # Show Travel Moves Toggle Button
         self.show_travel_button = Gtk.ToggleButton()
-        self.show_travel_button.set_child(get_icon("timeline"))
+        self.show_travel_button.set_child(get_icon("function-linear-symbolic"))
         self.show_travel_button.set_active(False)
         self.show_travel_button.set_tooltip_text(
             _("Toggle travel move visibility")
@@ -215,13 +242,13 @@ class MainWindow(Adw.ApplicationWindow):
         toolbar.append(sep)
 
         self.home_button = Gtk.Button()
-        self.home_button.set_child(get_icon("home"))
+        self.home_button.set_child(get_icon("go-home-symbolic"))
         self.home_button.set_tooltip_text(_("Home the machine"))
         self.home_button.connect("clicked", self.on_home_clicked)
         toolbar.append(self.home_button)
 
         self.frame_button = Gtk.Button()
-        self.frame_button.set_child(get_icon("frame"))
+        self.frame_button.set_child(get_icon("edit-select-all-symbolic"))
         self.frame_button.set_tooltip_text(
             _("Cycle laser head around the occupied area")
         )
@@ -229,13 +256,13 @@ class MainWindow(Adw.ApplicationWindow):
         toolbar.append(self.frame_button)
 
         self.send_button = Gtk.Button()
-        self.send_button.set_child(get_icon("send"))
+        self.send_button.set_child(get_icon("document-send-symbolic"))
         self.send_button.set_tooltip_text(_("Send to machine"))
         self.send_button.connect("clicked", self.on_send_clicked)
         toolbar.append(self.send_button)
 
-        self.hold_on_icon = get_icon("play-arrow")
-        self.hold_off_icon = get_icon("pause")
+        self.hold_on_icon = get_icon("pause-symbolic")
+        self.hold_off_icon = get_icon("pause-symbolic")
         self.hold_button = Gtk.ToggleButton()
         self.hold_button.set_child(self.hold_off_icon)
         self.hold_button.set_tooltip_text(_("Pause machine"))
@@ -243,13 +270,40 @@ class MainWindow(Adw.ApplicationWindow):
         toolbar.append(self.hold_button)
 
         self.cancel_button = Gtk.Button()
-        self.cancel_button.set_child(get_icon("stop"))
+        self.cancel_button.set_child(get_icon("process-stop-symbolic"))
         self.cancel_button.set_tooltip_text(_("Cancel running job"))
         self.cancel_button.connect("clicked", self.on_cancel_clicked)
         toolbar.append(self.cancel_button)
 
+        # Add spacer to push machine selector to the right
+        spacer = Gtk.Box()
+        spacer.set_hexpand(True)
+        toolbar.append(spacer)
+
+        # Add clickable warning for misconfigured machine
+        self.machine_warning_box = Gtk.Box(spacing=6)
+        self.machine_warning_box.set_margin_end(12)
+        warning_icon = Gtk.Image.new_from_icon_name("dialog-warning-symbolic")
+        warning_label = Gtk.Label(label=_("Machine not fully configured"))
+        warning_label.add_css_class("warning-label")
+        self.machine_warning_box.append(warning_icon)
+        self.machine_warning_box.append(warning_label)
+        self.machine_warning_box.set_tooltip_text(
+            _("Machine driver is missing required settings. Click to edit.")
+        )
+        self.machine_warning_box.set_visible(False)
+        warning_click = Gtk.GestureClick.new()
+        warning_click.connect("pressed", self._on_machine_warning_clicked)
+        self.machine_warning_box.add_controller(warning_click)
+        toolbar.append(self.machine_warning_box)
+
+        # Add machine selector dropdown
+        self._setup_machine_selector()
+        toolbar.append(self.machine_dropdown)
+
         # Create the Paned splitting the window into left and right sections.
         self.paned = Gtk.Paned(orientation=Gtk.Orientation.HORIZONTAL)
+        self.paned.set_vexpand(True)
         vbox.append(self.paned)
 
         # Apply styles
@@ -266,13 +320,13 @@ class MainWindow(Adw.ApplicationWindow):
         ratio = width_mm / height_mm
         self.frame = Gtk.AspectFrame(ratio=ratio, obey_child=False)
         self.frame.set_margin_start(12)
-        self.frame.set_margin_end(12)
         self.frame.set_hexpand(True)
         self.paned.set_start_child(self.frame)
 
         # Make a default document.
         self.doc = Doc()
         self.doc.changed.connect(self.on_doc_changed)
+        self.doc.active_layer_changed.connect(self._on_active_layer_changed)
         self.doc.history_manager.changed.connect(self.on_history_changed)
 
         self.surface = WorkSurface(
@@ -288,27 +342,43 @@ class MainWindow(Adw.ApplicationWindow):
         self.redo_button.set_history_manager(self.doc.history_manager)
 
         # Create a vertical paned for the right pane content
-        right_pane_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-        right_pane_box.set_margin_top(6)
-        right_pane_box.set_margin_bottom(12)
-        self.paned.set_end_child(right_pane_box)
+        right_pane_scrolled_window = Gtk.ScrolledWindow()
+        right_pane_scrolled_window.set_policy(
+            Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC
+        )
+        right_pane_scrolled_window.set_vexpand(True)
+        right_pane_scrolled_window.set_margin_start(10)
+        right_pane_scrolled_window.set_margin_top(6)
+        right_pane_scrolled_window.set_margin_bottom(12)
+        self.paned.set_end_child(right_pane_scrolled_window)
         self.paned.set_resize_end_child(False)
         self.paned.set_shrink_end_child(False)
 
-        # Show the work plan.
-        self.workplanview = WorkPlanView(self.doc, self.doc.workplan)
-        self.workplanview.set_size_request(400, -1)
-        self.workplanview.set_vexpand(True)
-        self.workplanview.set_margin_start(4)
-        right_pane_box.append(self.workplanview)
+        # Create a vertical box to organize the content within the
+        # ScrolledWindow.
+        right_pane_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        right_pane_scrolled_window.set_child(right_pane_box)
+
+        # Add the Layer list view
+        self.layer_list_view = LayerListView(self.doc)
+        self.layer_list_view.set_margin_end(12)
+        right_pane_box.append(self.layer_list_view)
+
+        # The WorkflowView will be updated when a layer is activated.
+        initial_workflow = self.doc.layers[0].workflow
+        self.workflowview = WorkflowView(initial_workflow)
+        self.workflowview.set_size_request(400, -1)
+        self.workflowview.set_margin_top(20)
+        self.workflowview.set_margin_end(12)
+        right_pane_box.append(self.workflowview)
 
         # Add the WorkpiecePropertiesWidget
         self.workpiece_props_widget = WorkpiecePropertiesWidget()
         workpiece_props_container = Gtk.Box(
             orientation=Gtk.Orientation.VERTICAL
         )
-        workpiece_props_container.set_vexpand(False)
-        workpiece_props_container.set_valign(Gtk.Align.END)
+        self.workpiece_props_widget.set_margin_top(20)
+        self.workpiece_props_widget.set_margin_end(12)
         workpiece_props_container.append(self.workpiece_props_widget)
 
         self.workpiece_revealer = Gtk.Revealer()
@@ -318,8 +388,6 @@ class MainWindow(Adw.ApplicationWindow):
             Gtk.RevealerTransitionType.SLIDE_UP
         )
         right_pane_box.append(self.workpiece_revealer)
-        self.workpiece_props_widget.set_margin_top(20)
-        self.workpiece_props_widget.set_margin_start(4)
 
         # Connect signals for workpiece selection
         self.surface.selection_changed.connect(self._on_selection_changed)
@@ -374,7 +442,102 @@ class MainWindow(Adw.ApplicationWindow):
         self.needs_homing = config.machine.home_on_start
 
         # Set initial state
-        self.update_state()
+        self.on_config_changed(None)
+
+    def _on_root_click_pressed(self, gesture, n_press, x, y):
+        """
+        Global click handler to unfocus widgets when clicking on "dead space".
+        """
+        self.surface.grab_focus()
+
+    def _setup_machine_selector(self):
+        """Creates the Gtk.DropDown for machine selection."""
+        # The model holds GObject wrappers for our Machine objects.
+        self.machine_model = Gio.ListStore.new(MachineListItem)
+
+        # The Factory for the list items in the popup
+        factory = Gtk.SignalListItemFactory()
+        factory.connect("setup", self._on_machine_selector_setup)
+        factory.connect("bind", self._on_machine_selector_bind)
+
+        # The Expression for the main dropdown button
+        expression = Gtk.ClosureExpression.new(
+            str, lambda item: item.machine.name if item else "", None
+        )
+
+        # Create the DropDown and set the factory
+        self.machine_dropdown = Gtk.DropDown.new(
+            self.machine_model, expression
+        )
+        self.machine_dropdown.set_factory(factory)
+
+        self.machine_dropdown.set_tooltip_text(_("Select active machine"))
+        self.machine_dropdown.connect(
+            "notify::selected-item", self._on_machine_selected
+        )
+
+        # Connect to the machine manager to keep the list updated.
+        machine_mgr.machine_added.connect(self._update_machine_list)
+        machine_mgr.machine_removed.connect(self._update_machine_list)
+        machine_mgr.machine_updated.connect(self._update_machine_list)
+
+        # Initial population of the list.
+        self._update_machine_list()
+
+    def _on_machine_selector_setup(self, factory, list_item):
+        """Setup a list item for the machine dropdown."""
+        box = Gtk.Box(spacing=6)
+        label = Gtk.Label()
+        box.append(label)
+        list_item.set_child(box)
+
+    def _on_machine_selector_bind(self, factory, list_item):
+        """Bind a machine object to a list item."""
+        box = list_item.get_child()
+        label = box.get_first_child()
+        list_item_obj = list_item.get_item()
+        label.set_text(list_item_obj.machine.name)
+
+    def _update_machine_list(self, *args, **kwargs):
+        """
+        Repopulates the machine dropdown from the machine manager.
+        """
+        logger.debug("Updating machine list in dropdown.")
+        machines = sorted(machine_mgr.machines.values(), key=lambda m: m.name)
+
+        # Block the selection signal while we modify the list to avoid
+        # recursion.
+        self.machine_dropdown.handler_block_by_func(self._on_machine_selected)
+
+        try:
+            self.machine_model.remove_all()
+            selected_index = -1
+            for i, machine in enumerate(machines):
+                # Append an instance of the wrapper, not the raw object
+                self.machine_model.append(MachineListItem(machine))
+                if config.machine and machine.id == config.machine.id:
+                    selected_index = i
+
+            if selected_index != -1:
+                self.machine_dropdown.set_selected(selected_index)
+        finally:
+            # Unblock the signal in a finally block to ensure it always runs.
+            self.machine_dropdown.handler_unblock_by_func(
+                self._on_machine_selected
+            )
+
+        # Set visibility based on the number of machines
+        self.machine_dropdown.set_visible(len(machines) > 1)
+
+    def _on_machine_selected(self, dropdown, param):
+        """Handles when a user selects a new machine from the dropdown."""
+        selected_list_item = dropdown.get_selected_item()
+        if selected_list_item:
+            # selected_list_item is the MachineListItem wrapper
+            logger.info(
+                f"User selected machine: {selected_list_item.machine.name}"
+            )
+            config.set_machine(selected_list_item.machine)
 
     def _setup_actions(self):
         """Creates all Gio.SimpleActions for the window and application."""
@@ -424,9 +587,9 @@ class MainWindow(Adw.ApplicationWindow):
         self.remove_action.connect("activate", self.on_menu_remove)
         self.add_action(self.remove_action)
 
-        settings_action = Gio.SimpleAction.new("settings", None)
-        settings_action.connect("activate", self.show_machine_settings)
-        self.add_action(settings_action)
+        preferences_action = Gio.SimpleAction.new("preferences", None)
+        preferences_action.connect("activate", self.show_preferences)
+        self.add_action(preferences_action)
 
         # Help action
         about_action = Gio.SimpleAction.new("about", None)
@@ -461,7 +624,7 @@ class MainWindow(Adw.ApplicationWindow):
         edit_menu.append_section(None, clipboard_commands)
 
         other_edit_commands = Gio.Menu()
-        other_edit_commands.append(_("Preferences"), "win.settings")
+        other_edit_commands.append(_("Preferences…"), "win.preferences")
         edit_menu.append_section(None, other_edit_commands)
 
         menu_model.append_submenu(_("_Edit"), edit_menu)
@@ -494,7 +657,7 @@ class MainWindow(Adw.ApplicationWindow):
         app.set_accels_for_action("win.paste", ["<Primary>v"])
         app.set_accels_for_action("win.duplicate", ["<Primary>d"])
         app.set_accels_for_action("win.remove", ["Delete"])
-        app.set_accels_for_action("win.settings", ["<Primary>comma"])
+        app.set_accels_for_action("win.preferences", ["<Primary>comma"])
         app.set_accels_for_action("win.about", ["F1"])
 
     def _try_driver_setup(self):
@@ -542,11 +705,24 @@ class MainWindow(Adw.ApplicationWindow):
 
     def on_history_changed(self, history_manager):
         self.update_state()
+        # After undo/redo, the document state may have changed in ways
+        # that require a full UI sync (e.g., layer visibility).
+        self.on_doc_changed(self.doc)
 
     def on_doc_changed(self, sender, **kwargs):
+        # Synchronize UI elements that depend on the document model
         self.surface.update_from_doc(self.doc)
+        if self.doc.active_layer:
+            self.workflowview.set_workflow(self.doc.active_layer.workflow)
+
+        # Update button sensitivity and other state
         self.update_state()
 
+    def _on_active_layer_changed(self, sender):
+        """Resets the paste counter when the active layer changes."""
+        self._paste_counter = 0
+        logger.debug("Active layer changed, paste counter reset.")
+
     def _on_selection_changed(
         self,
         sender,
@@ -576,7 +752,7 @@ class MainWindow(Adw.ApplicationWindow):
         self.update_state()
 
     def on_config_changed(self, sender, **kwargs):
-        self.surface.set_size(*config.machine.dimensions)
+        self.surface.set_machine(config.machine)
         width_mm, height_mm = config.machine.dimensions
         self.frame.set_ratio(width_mm / height_mm)
 
@@ -585,6 +761,22 @@ class MainWindow(Adw.ApplicationWindow):
         self.surface.update_from_doc(self.doc)
         self.update_state()
 
+        # Update theme
+        self.apply_theme()
+
+        # Ensure dropdown selection is synced with config
+        self._update_machine_list()
+
+    def apply_theme(self):
+        """Reads the theme from config and applies it to the UI."""
+        style_manager = Adw.StyleManager.get_default()
+        if config.theme == "light":
+            style_manager.set_color_scheme(Adw.ColorScheme.FORCE_LIGHT)
+        elif config.theme == "dark":
+            style_manager.set_color_scheme(Adw.ColorScheme.FORCE_DARK)
+        else:  # "system" or any other invalid value
+            style_manager.set_color_scheme(Adw.ColorScheme.DEFAULT)
+
     def on_running_tasks_changed(self, sender, tasks, progress):
         self.update_state()
 
@@ -597,6 +789,12 @@ class MainWindow(Adw.ApplicationWindow):
         can_redo = self.doc.history_manager.can_redo()
         can_paste = len(self._clipboard_snapshot) > 0
 
+        # Show warning if the current machine is not configured
+        warning_visible = False
+        if driver_mgr.driver and not driver_mgr.driver.did_setup:
+            warning_visible = True
+        self.machine_warning_box.set_visible(warning_visible)
+
         # Update action sensitivity
         self.export_action.set_enabled(can_export)
         self.undo_action.set_enabled(can_undo)
@@ -636,6 +834,9 @@ class MainWindow(Adw.ApplicationWindow):
         if driver_mgr.driver.__class__ is NoDeviceDriver:
             send_tooltip = _("Send to machine (select driver to enable)")
             send_sensitive = False
+        elif not (driver_mgr.driver and driver_mgr.driver.did_setup):
+            send_tooltip = _("Send to machine (configure driver to enable)")
+            send_sensitive = False
         elif conn_status != TransportStatus.CONNECTED:
             send_tooltip = _("Send to machine (connect to enable)")
             send_sensitive = False
@@ -665,13 +866,30 @@ class MainWindow(Adw.ApplicationWindow):
         connected = conn_status == TransportStatus.CONNECTED
         self.surface.set_laser_dot_visible(connected)
         state = self.machine_status.state
-        if state and None not in state.machine_pos:
-            self.surface.set_laser_dot_position(*state.machine_pos[:2])
+        if not state:
+            return
+        x, y = state.machine_pos[:2]
+        if x is not None and y is not None:
+            self.surface.set_laser_dot_position(x, y)
+
+    def _on_machine_warning_clicked(self, *args):
+        """Opens the machine settings dialog for the current machine."""
+        current_machine = config.machine
+        if not current_machine:
+            return
+
+        dialog = MachineSettingsDialog(machine=current_machine)
+        dialog.present(self)
 
     def on_status_bar_clicked(self, gesture, n_press, x, y, box):
         dialog = MachineView()
+        dialog.notification_requested.connect(self._on_dialog_notification)
         dialog.present(self)
 
+    def _on_dialog_notification(self, sender, message: str = ""):
+        """Shows a toast when requested by a child dialog."""
+        self.toast_overlay.add_toast(Adw.Toast.new(message))
+
     def on_quit_action(self, action, parameter):
         self.close()
 
@@ -770,7 +988,7 @@ class MainWindow(Adw.ApplicationWindow):
                 if not head.frame_power:
                     return
 
-                ops = await self.doc.workplan.execute(context)
+                ops = await self.doc.generate_job_ops(context)
                 frame = ops.get_frame(
                     power=head.frame_power,
                     speed=config.machine.max_travel_speed,
@@ -794,7 +1012,7 @@ class MainWindow(Adw.ApplicationWindow):
 
         async def send_coro(context: ExecutionContext):
             try:
-                ops = await self.doc.workplan.execute(context)
+                ops = await self.doc.generate_job_ops(context)
                 if not driver_mgr.driver:
                     raise RuntimeError("No driver configured to send job.")
                 await driver_mgr.driver.run(ops, config.machine)
@@ -840,11 +1058,11 @@ class MainWindow(Adw.ApplicationWindow):
         async def export_coro(context: ExecutionContext):
             try:
                 # 1. Generate Ops (async, reports progress)
-                ops = await self.doc.workplan.execute(context)
+                ops = await self.doc.generate_job_ops(context)
 
                 # 2. Encode G-code (sync, but usually fast)
                 context.set_message("Encoding G-code...")
-                encoder = GcodeEncoder()
+                encoder = GcodeEncoder.for_machine(config.machine)
                 gcode = encoder.encode(ops, config.machine)
 
                 # 3. Write to file (sync, potentially slow, run in thread)
@@ -900,9 +1118,33 @@ class MainWindow(Adw.ApplicationWindow):
                 return
 
         wp = WorkPiece.from_file(filename, renderer)
+
+        # Calculate and set a default size and position for the new workpiece
+        if wp.pos is None or wp.size is None:
+            wswidth_mm, wsheight_mm = self.surface.get_size()
+            wp_width_nat_mm, wp_height_nat_mm = wp.get_default_size(
+                wswidth_mm, wsheight_mm
+            )
+
+            # Determine the size to use in mm, scaling down if necessary to fit
+            width_mm = wp_width_nat_mm
+            height_mm = wp_height_nat_mm
+            if width_mm > wswidth_mm or height_mm > wsheight_mm:
+                scale_w = wswidth_mm / width_mm if width_mm > 0 else 1
+                scale_h = wsheight_mm / height_mm if height_mm > 0 else 1
+                scale = min(scale_w, scale_h)
+                width_mm *= scale
+                height_mm *= scale
+
+            # Set the workpiece's size and centered position in mm
+            wp.set_size(width_mm, height_mm)
+            x_mm = (wswidth_mm - width_mm) / 2
+            y_mm = (wsheight_mm - height_mm) / 2
+            wp.set_pos(x_mm, y_mm)
+
         cmd_name = _("Import {name}").format(name=filename.name)
         command = ListItemCommand(
-            owner_obj=self.doc,
+            owner_obj=self.doc.active_layer,
             item=wp,
             undo_command="remove_workpiece",
             redo_command="add_workpiece",
@@ -920,6 +1162,9 @@ class MainWindow(Adw.ApplicationWindow):
             return
 
         self.on_copy_requested(sender, workpieces)
+        # For a cut, the next paste should be at the original location
+        # (no offset).
+        self._paste_counter = 0
 
         history = self.doc.history_manager
         with history.transaction(_("Cut workpiece(s)")) as t:
@@ -943,32 +1188,34 @@ class MainWindow(Adw.ApplicationWindow):
             return
         # Create a snapshot of the current state by serializing to dicts.
         self._clipboard_snapshot = [wp.to_dict() for wp in workpieces]
-        # Reset the paste counter for a new copy/paste sequence.
-        self._paste_counter = 0
+        # For a copy, the next paste should be offset.
+        self._paste_counter = 1
         logger.debug(
             f"Copied {len(self._clipboard_snapshot)} workpieces. "
-            "Paste counter reset."
+            "Paste counter set to 1."
         )
 
     def on_paste_requested(self, sender, *args):
         """
         Handles the 'paste-requested' signal. Pastes a new set of items
         with a cumulative offset from the original clipboard snapshot.
+        For a cut operation, the first paste is at the original location.
         """
         if not self._clipboard_snapshot:
             return
 
-        self._paste_counter += 1
         history = self.doc.history_manager
         newly_pasted_workpieces = []
 
         with history.transaction(_("Paste workpiece(s)")) as t:
+            # The paste counter determines the offset level.
+            # It's 0 for the first paste of a cut, and >0 for all others.
             offset_x = self._paste_increment_mm[0] * self._paste_counter
             offset_y = self._paste_increment_mm[1] * self._paste_counter
 
             for wp_dict in self._clipboard_snapshot:
                 new_wp = WorkPiece.from_dict(wp_dict)
-                new_wp.uid = uuid.uuid4()
+                new_wp.uid = str(uuid.uuid4())
                 newly_pasted_workpieces.append(new_wp)
 
                 original_pos = wp_dict.get("pos")
@@ -979,7 +1226,7 @@ class MainWindow(Adw.ApplicationWindow):
 
                 cmd_name = _("Paste {name}").format(name=new_wp.name)
                 command = ListItemCommand(
-                    owner_obj=self.doc,
+                    owner_obj=self.doc.active_layer,
                     item=new_wp,
                     undo_command="remove_workpiece",
                     redo_command="add_workpiece",
@@ -987,6 +1234,9 @@ class MainWindow(Adw.ApplicationWindow):
                 )
                 t.execute(command)
 
+        # Increment the counter for the *next* paste operation.
+        self._paste_counter += 1
+
         if newly_pasted_workpieces:
             self.surface.select_workpieces(newly_pasted_workpieces)
 
@@ -1005,7 +1255,7 @@ class MainWindow(Adw.ApplicationWindow):
             for wp in workpieces:
                 wp_dict = wp.to_dict()
                 new_wp = WorkPiece.from_dict(wp_dict)
-                new_wp.uid = uuid.uuid4()
+                new_wp.uid = str(uuid.uuid4())
                 newly_duplicated_workpieces.append(new_wp)
 
                 cmd_name = _("Duplicate {name}").format(name=new_wp.name)
@@ -1057,11 +1307,11 @@ class MainWindow(Adw.ApplicationWindow):
         dialog = AboutDialog(transient_for=self)
         dialog.present()
 
-    def show_machine_settings(self, action, param):
-        dialog = MachineSettingsDialog(config.machine)
-        dialog.present(self)
-        dialog.connect("closed", self._on_settings_dialog_closed)
+    def show_preferences(self, action, param):
+        dialog = PreferencesWindow(transient_for=self)
+        dialog.present()
+        dialog.connect("close-request", self._on_preferences_dialog_closed)
 
-    def _on_settings_dialog_closed(self, dialog):
-        logger.debug("Settings closed")
+    def _on_preferences_dialog_closed(self, dialog):
+        logger.debug("Preferences dialog closed")
         self.surface.grab_focus()  # re-enables keyboard shortcuts
